# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: developers@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import os

from patch_api.api_client import ApiClient


class TestEstimatesApi(unittest.TestCase):
    """EstimatesApi unit test stubs"""

    def setUp(self):
        api_client = ApiClient(api_key=os.environ.get("SANDBOX_API_KEY"))
        self.api = api_client.estimates  # noqa: E501

    def tearDown(self):
        self.api = None

    def test_create_and_retrieve_mass_estimate(self):
        """Test case for create_mass_estimate

        Create an estimate based on mass of CO2  # noqa: E501
        """
        mass_g = 100
        project_id = "pro_test_2b67b11a030b66e0a6dd61a56b49079a"
        estimate = self.api.create_mass_estimate(
            mass_g=mass_g, project_id=project_id, create_order=True
        )
        self.assertTrue(estimate)
        self.assertEqual(estimate.data.order.mass_g, mass_g)

        retrieved_estimate = self.api.retrieve_estimate(id=estimate.data.id)
        self.assertTrue(retrieved_estimate)

    def test_create_and_retrieve_flight_estimate(self):
        """Test case for create_flight_estimate

        Create an estimate based on the distance in meters flown by an airplane # noqa: E501
        """
        distance_m = 1000000
        estimate = self.api.create_flight_estimate(
            distance_m=distance_m, create_order=True
        )
        self.assertEqual(estimate.data.type, "flight")
        self.assertGreater(estimate.data.order.mass_g, 50000)
        self.assertGreater(estimate.data.mass_g, 50000)

        retrieved_estimate = self.api.retrieve_estimate(id=estimate.data.id)
        self.assertTrue(retrieved_estimate)

    def test_create_and_retrieve_flight_estimate_with_airports(self):
        """Test case for create_flight_estimate

        Create an estimate based on the distance in meters flown by an airplane # noqa: E501
        """
        estimate_short = self.api.create_flight_estimate(
            origin_airport="SFO", destination_airport="LAX"
        )
        estimate_long = self.api.create_flight_estimate(
            origin_airport="SFO", destination_airport="JFK"
        )
        self.assertGreater(estimate_long.data.mass_g, estimate_short.data.mass_g)

    def test_create_and_retrieve_shipping_estimate(self):
        """Test case for create_shipping_estimate

        Create an estimate based on the shipping distance, transportation method, and package mass  # noqa: E501
        """
        distance_m = 10000000
        package_mass_g = 1000
        transportation_method = "sea"
        estimate = self.api.create_shipping_estimate(
            distance_m=distance_m,
            package_mass_g=package_mass_g,
            transportation_method=transportation_method,
            create_order=False,
        )
        self.assertEqual(estimate.data.order, None)
        self.assertEqual(estimate.data.type, "shipping")
        self.assertGreater(estimate.data.mass_g, 200)

        retrieved_estimate = self.api.retrieve_estimate(id=estimate.data.id)
        self.assertTrue(retrieved_estimate)

    def test_create_and_retrieve_vehicle_estimate(self):
        """Test case for create_vehicle_estimate

        Create an estimate based on the vehicle distance, transportation method, and package mass  # noqa: E501
        """
        distance_m = 1000000
        make = "Toyota"
        model = "Corolla"
        year = 1995
        estimate = self.api.create_vehicle_estimate(
            distance_m=distance_m, model=model, make=make, year=year
        )
        self.assertEqual(estimate.data.type, "vehicle")
        self.assertGreater(estimate.data.mass_g, 50000)

        retrieved_estimate = self.api.retrieve_estimate(id=estimate.data.id)
        self.assertTrue(retrieved_estimate)

    def test_create_and_retrieve_vehicle_estimate_best_match(self):
        """Test case for create_vehicle_estimate

        Create an estimate based on the vehicle with partial information  # noqa: E501
        """
        distance_m = 1000000
        make = "Toyota"
        model = "Corolla"
        estimate = self.api.create_vehicle_estimate(
            distance_m=distance_m, model=model, make=make
        )
        self.assertEqual(estimate.data.type, "vehicle")
        self.assertGreater(estimate.data.mass_g, 50000)

        retrieved_estimate = self.api.retrieve_estimate(id=estimate.data.id)
        self.assertTrue(retrieved_estimate)

    def test_create_bitcoin_estimate_no_params(self):
        """Test case for create_bitcoin_estimate

        Create an estimate based on a transaction amount  # noqa: E501
        """

        estimate = self.api.create_bitcoin_estimate()
        self.assertEqual(estimate.data.type, "bitcoin")
        self.assertGreater(
            estimate.data.mass_g, 200
        )  # not setting an exact value since this is changing daily

    def test_create_bitcoin_estimate_transaction_value(self):
        """Test case for create_bitcoin_estimate

        Create an estimate based on a transaction amount  # noqa: E501
        """
        transaction_value_btc_sats = 100000

        estimate = self.api.create_bitcoin_estimate(
            transaction_value_btc_sats=transaction_value_btc_sats
        )
        self.assertEqual(estimate.data.type, "bitcoin")
        self.assertGreater(
            estimate.data.mass_g, 200
        )  # not setting an exact value since this is changing daily

    def test_create_bitcoin_estimate_transaction_value(self):
        """Test case for create_bitcoin_estimate

        Create an estimate based on an average daily balance  # noqa: E501
        """
        estimate1 = self.api.create_bitcoin_estimate(
            average_daily_balance_btc_sats=100000
        )
        estimate2 = self.api.create_bitcoin_estimate(
            average_daily_balance_btc_sats=1000000
        )
        self.assertEqual(estimate1.data.type, "bitcoin")
        self.assertGreater(estimate2.data.mass_g, estimate1.data.mass_g)

    def test_create_ethereum_estimate_transaction_value(self):
        """Test case for create_ethereum_estimate

        Create an estimate based on a transaction amount  # noqa: E501
        """

        estimate = self.api.create_ethereum_estimate(gas_used=1000)
        self.assertEqual(estimate.data.type, "ethereum")
        self.assertGreater(
            estimate.data.mass_g, 1
        )  # not setting an exact value since this is changing daily

    def test_create_hotel_estimate(self):
        """Test case for create_hotel_estimate

        Create an estimate based on a country code  # noqa: E501
        """

        estimate = self.api.create_hotel_estimate(country_code="US")
        self.assertEqual(estimate.data.type, "hotel")
        self.assertGreater(
            estimate.data.mass_g, 15_000
        )  # not setting an exact value since this is changing daily


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: developers@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import os

from patch_api.api_client import ApiClient


class TestProjectsApi(unittest.TestCase):
    """ProjectsApi unit test stubs"""

    def setUp(self):
        api_client = ApiClient(api_key=os.environ.get("SANDBOX_API_KEY"))
        self.api = api_client.projects  # noqa: E501

    def tearDown(self):
        self.api = None

    def test_retrieve_project(self):
        """Test case for retrieve_project

        Retrieves a project  # noqa: E501
        """
        project_id = "pro_test_2b67b11a030b66e0a6dd61a56b49079a"
        project = self.api.retrieve_project(id=project_id).data
        self.assertTrue(project)
        self.assertEqual(project.production, False)
        self.assertTrue(isinstance(project.photos, list))

        self.assertTrue(hasattr(project, "tagline"))
        self.assertTrue(hasattr(project, "latitude"))
        self.assertTrue(hasattr(project, "longitude"))
        self.assertTrue(hasattr(project, "highlights"))

        self.assertTrue(isinstance(project.mechanism, str))
        self.assertTrue(isinstance(project.state, str))

        technology_type = project.technology_type
        self.assertTrue(isinstance(technology_type.name, str))
        self.assertTrue(isinstance(technology_type.slug, str))

        parent_technology_type = technology_type.parent_technology_type
        self.assertTrue(isinstance(parent_technology_type.name, str))
        self.assertTrue(isinstance(parent_technology_type.slug, str))

    def test_retrieve_projects(self):
        """Test case for retrieve_projects

        Retrieves a list of projects  # noqa: E501
        """
        projects = self.api.retrieve_projects().data
        self.assertTrue(isinstance(projects, list))

        if len(projects) > 0:
            project = projects[0]

            self.assertEqual(project.production, False)
            self.assertGreater(project.average_price_per_tonne_cents_usd, 0)
            self.assertGreater(project.remaining_mass_g, 0)
            self.assertEqual(project.standard, None)
            self.assertIsInstance(project.name, str)
            self.assertTrue(project.description)
            self.assertIsInstance(project.country, str)
            self.assertIsInstance(project.type, str)
            self.assertIsInstance(project.developer, str)
            self.assertTrue(isinstance(project.photos, list))

    def test_retrieve_biomass_projects(self):
        """Test case for retrieve_projects with a type filter

        Retrieves a list of projects  # noqa: E501
        """
        project_type = "biomass"
        projects = self.api.retrieve_projects(type=project_type).data
        self.assertTrue(isinstance(projects, list))

        for project in projects:
            self.assertEqual(project.type, project_type)

    def test_retrieve_american_projects(self):
        """Test case for retrieve_projects with a country filter

        Retrieves a list of projects  # noqa: E501
        """
        project_country = "US"
        projects = self.api.retrieve_projects(country=project_country).data
        self.assertTrue(isinstance(projects, list))

        for project in projects:
            self.assertEqual(project.country, project_country)

    def test_retrieve_projects_with_more_than_100_grams_of_inventory(self):
        """Test case for retrieve_projects with a country filter

        Retrieves a list of projects  # noqa: E501
        """
        minimum_available_mass = 100
        projects = self.api.retrieve_projects(
            minimum_available_mass=minimum_available_mass
        ).data
        self.assertTrue(isinstance(projects, list))

        for project in projects:
            self.assertTrue(project.remaining_mass_g >= minimum_available_mass)


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: developers@usepatch.com
    Generated by: https://openapi-generator.tech
"""

from __future__ import absolute_import

import unittest
import os
import datetime

from patch_api.api_client import ApiClient


class TestOrdersApi(unittest.TestCase):
    """OrdersApi unit test stubs"""

    def setUp(self):
        api_client = ApiClient(api_key=os.environ.get("SANDBOX_API_KEY"))
        self.api = api_client.orders  # noqa: E501

    def tearDown(self):
        self.api = None

    def test_interactions_with_an_order(self):
        """Test case for create_order"""

        """Create an order
        """
        order = self.api.create_order(mass_g=100)

        self.assertTrue(order)

        self.assertIsInstance(order.data.created_at, datetime.datetime)

        self.assertEqual(order.data.mass_g, 100)

        """Create an order on price
        """
        order = self.api.create_order(total_price_cents_usd=100)

        self.assertTrue(order)
        self.assertEqual(
            order.data.price_cents_usd + order.data.patch_fee_cents_usd, 100
        )

        """Create an order in draft state
        """
        order = self.api.create_order(mass_g=100, state="draft")

        self.assertTrue(order)
        self.assertEqual(order.data.mass_g, 100)
        self.assertEqual(order.data.state, "draft")

    def test_retrieve_order(self):
        """Test case for retrieve_order"""

        """Retrieve an order
        """
        order = self.api.create_order(mass_g=100)
        retrieved_order = self.api.retrieve_order(id=order.data.id)

        self.assertTrue(retrieved_order)
        self.assertEqual(retrieved_order.data.mass_g, 100)

    def test_retrieve_orders(self):
        """Test case for retrieve_orders

        Retrieves a list of orders  # noqa: E501
        """
        orders = self.api.retrieve_orders().data
        self.assertTrue(isinstance(orders, list))

        if len(orders) > 1:
            retrieved_order = orders[0]

            self.assertTrue(retrieved_order.id)
            self.assertEqual(retrieved_order.production, False)
            self.assertEqual(retrieved_order.state, "placed")
            self.assertEqual(retrieved_order.metadata, {})
            self.assertTrue(isinstance(retrieved_order.allocations, list))

    def test_create_and_retrieve_order_via_metadata(self):
        """Test case for create_order and retrieve_orders with metadata"""
        mass_g = 100
        metadata = {"external_id": "abc-123"}
        order = self.api.create_order(mass_g=mass_g, metadata=metadata)

        self.assertTrue(order)
        self.assertEqual(order.data.mass_g, 100)
        self.assertEqual(order.data.metadata, {"external_id": "abc-123"})

        retrieved_orders = self.api.retrieve_orders(
            metadata={"external_id": "abc-"}
        ).data
        self.assertGreater(len(retrieved_orders), 0)
        for retrieved_order in retrieved_orders:
            self.assertTrue("external_id" in retrieved_order.metadata)

    def test_cancel_order_in_draft_state(self):
        """Test case for cancel_order on draft orders"""
        order = self.api.create_order(mass_g=100, state="draft")
        self.assertEqual(order.data.state, "draft")

        cancelled_order = self.api.cancel_order(id=order.data.id)
        self.assertEqual(cancelled_order.data.state, "cancelled")

    def test_create_order_with_vintage_year(self):
        """Test case for vintage_year on create order"""
        order = self.api.create_order(mass_g=100, vintage_year=2022)

        self.assertTrue(order)


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class CreateMassEstimateRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"mass_g": "int", "create_order": "bool", "project_id": "str"}

    attribute_map = {
        "mass_g": "mass_g",
        "create_order": "create_order",
        "project_id": "project_id",
    }

    def __init__(
        self,
        mass_g=None,
        create_order=False,
        project_id=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """CreateMassEstimateRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._mass_g = None
        self._create_order = None
        self._project_id = None
        self.discriminator = None

        self.mass_g = mass_g
        self.create_order = create_order
        if project_id is not None:
            self.project_id = project_id

    @property
    def mass_g(self):
        """Gets the mass_g of this CreateMassEstimateRequest.  # noqa: E501


        :return: The mass_g of this CreateMassEstimateRequest.  # noqa: E501
        :rtype: int
        """
        return self._mass_g

    @mass_g.setter
    def mass_g(self, mass_g):
        """Sets the mass_g of this CreateMassEstimateRequest.


        :param mass_g: The mass_g of this CreateMassEstimateRequest.  # noqa: E501
        :type: int
        """
        if (
            self.local_vars_configuration.client_side_validation and mass_g is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must not be `None`"
            )  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and mass_g is not None
            and mass_g > 100000000000
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must be a value less than or equal to `100000000000`"
            )  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and mass_g is not None
            and mass_g < 0
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must be a value greater than or equal to `0`"
            )  # noqa: E501

        self._mass_g = mass_g

    @property
    def create_order(self):
        """Gets the create_order of this CreateMassEstimateRequest.  # noqa: E501


        :return: The create_order of this CreateMassEstimateRequest.  # noqa: E501
        :rtype: bool
        """
        return self._create_order

    @create_order.setter
    def create_order(self, create_order):
        """Sets the create_order of this CreateMassEstimateRequest.


        :param create_order: The create_order of this CreateMassEstimateRequest.  # noqa: E501
        :type: bool
        """

        self._create_order = create_order

    @property
    def project_id(self):
        """Gets the project_id of this CreateMassEstimateRequest.  # noqa: E501


        :return: The project_id of this CreateMassEstimateRequest.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this CreateMassEstimateRequest.


        :param project_id: The project_id of this CreateMassEstimateRequest.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateMassEstimateRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateMassEstimateRequest):
            return True

        return self.to_dict() != other.to_dict()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from patch_api.api.estimates_api import EstimatesApi
from patch_api.api.orders_api import OrdersApi
from patch_api.api.projects_api import ProjectsApi
from patch_api.api.technology_types_api import TechnologyTypesApi



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class Project(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "str",
        "production": "bool",
        "name": "str",
        "description": "str",
        "type": "str",
        "mechanism": "str",
        "country": "str",
        "state": "str",
        "latitude": "float",
        "longitude": "float",
        "developer": "str",
        "photos": "list[Photo]",
        "average_price_per_tonne_cents_usd": "int",
        "remaining_mass_g": "int",
        "verifier": "str",
        "standard": "Standard",
        "sdgs": "list[Sdg]",
        "tagline": "str",
        "technology_type": "TechnologyType",
        "highlights": "list[Highlight]",
    }

    attribute_map = {
        "id": "id",
        "production": "production",
        "name": "name",
        "description": "description",
        "type": "type",
        "mechanism": "mechanism",
        "country": "country",
        "state": "state",
        "latitude": "latitude",
        "longitude": "longitude",
        "developer": "developer",
        "photos": "photos",
        "average_price_per_tonne_cents_usd": "average_price_per_tonne_cents_usd",
        "remaining_mass_g": "remaining_mass_g",
        "verifier": "verifier",
        "standard": "standard",
        "sdgs": "sdgs",
        "tagline": "tagline",
        "technology_type": "technology_type",
        "highlights": "highlights",
    }

    def __init__(
        self,
        id=None,
        production=None,
        name=None,
        description=None,
        type=None,
        mechanism=None,
        country=None,
        state=None,
        latitude=None,
        longitude=None,
        developer=None,
        photos=None,
        average_price_per_tonne_cents_usd=None,
        remaining_mass_g=None,
        verifier=None,
        standard=None,
        sdgs=None,
        tagline=None,
        technology_type=None,
        highlights=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """Project - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._production = None
        self._name = None
        self._description = None
        self._type = None
        self._mechanism = None
        self._country = None
        self._state = None
        self._latitude = None
        self._longitude = None
        self._developer = None
        self._photos = None
        self._average_price_per_tonne_cents_usd = None
        self._remaining_mass_g = None
        self._verifier = None
        self._standard = None
        self._sdgs = None
        self._tagline = None
        self._technology_type = None
        self._highlights = None
        self.discriminator = None

        self.id = id
        self.production = production
        self.name = name
        self.description = description
        if type is not None:
            self.type = type
        if mechanism is not None:
            self.mechanism = mechanism
        self.country = country
        self.state = state
        self.latitude = latitude
        self.longitude = longitude
        self.developer = developer
        self.photos = photos
        self.average_price_per_tonne_cents_usd = average_price_per_tonne_cents_usd
        self.remaining_mass_g = remaining_mass_g
        if verifier is not None:
            self.verifier = verifier
        self.standard = standard
        self.sdgs = sdgs
        if tagline is not None:
            self.tagline = tagline
        self.technology_type = technology_type
        self.highlights = highlights

    @property
    def id(self):
        """Gets the id of this Project.  # noqa: E501

        A unique uid for the record. UIDs will be prepended by pro_prod or pro_test depending on the mode it was created in.  # noqa: E501

        :return: The id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Project.

        A unique uid for the record. UIDs will be prepended by pro_prod or pro_test depending on the mode it was created in.  # noqa: E501

        :param id: The id of this Project.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and id is None
        ):  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def production(self):
        """Gets the production of this Project.  # noqa: E501

        A boolean indicating if this project is a production or test mode project.  # noqa: E501

        :return: The production of this Project.  # noqa: E501
        :rtype: bool
        """
        return self._production

    @production.setter
    def production(self, production):
        """Sets the production of this Project.

        A boolean indicating if this project is a production or test mode project.  # noqa: E501

        :param production: The production of this Project.  # noqa: E501
        :type: bool
        """
        if (
            self.local_vars_configuration.client_side_validation and production is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `production`, must not be `None`"
            )  # noqa: E501

        self._production = production

    @property
    def name(self):
        """Gets the name of this Project.  # noqa: E501

        The name of the project.  # noqa: E501

        :return: The name of this Project.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Project.

        The name of the project.  # noqa: E501

        :param name: The name of this Project.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and name is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def description(self):
        """Gets the description of this Project.  # noqa: E501

        The description of the project.  # noqa: E501

        :return: The description of this Project.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Project.

        The description of the project.  # noqa: E501

        :param description: The description of this Project.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and description is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `description`, must not be `None`"
            )  # noqa: E501

        self._description = description

    @property
    def type(self):
        """Gets the type of this Project.  # noqa: E501

        DEPRECATED. Favor the technology_type field instead.  # noqa: E501

        :return: The type of this Project.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Project.

        DEPRECATED. Favor the technology_type field instead.  # noqa: E501

        :param type: The type of this Project.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def mechanism(self):
        """Gets the mechanism of this Project.  # noqa: E501

        The mechanism of the project. Either removal or avoidance.  # noqa: E501

        :return: The mechanism of this Project.  # noqa: E501
        :rtype: str
        """
        return self._mechanism

    @mechanism.setter
    def mechanism(self, mechanism):
        """Sets the mechanism of this Project.

        The mechanism of the project. Either removal or avoidance.  # noqa: E501

        :param mechanism: The mechanism of this Project.  # noqa: E501
        :type: str
        """
        allowed_values = ["removal", "avoidance"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and mechanism not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mechanism` ({0}), must be one of {1}".format(  # noqa: E501
                    mechanism, allowed_values
                )
            )

        self._mechanism = mechanism

    @property
    def country(self):
        """Gets the country of this Project.  # noqa: E501

        The country of origin of the project.  # noqa: E501

        :return: The country of this Project.  # noqa: E501
        :rtype: str
        """
        return self._country

    @country.setter
    def country(self, country):
        """Sets the country of this Project.

        The country of origin of the project.  # noqa: E501

        :param country: The country of this Project.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and country is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `country`, must not be `None`"
            )  # noqa: E501

        self._country = country

    @property
    def state(self):
        """Gets the state of this Project.  # noqa: E501

        The state where this project is located.  # noqa: E501

        :return: The state of this Project.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this Project.

        The state where this project is located.  # noqa: E501

        :param state: The state of this Project.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def latitude(self):
        """Gets the latitude of this Project.  # noqa: E501

        The latitude at which this project is located.  # noqa: E501

        :return: The latitude of this Project.  # noqa: E501
        :rtype: float
        """
        return self._latitude

    @latitude.setter
    def latitude(self, latitude):
        """Sets the latitude of this Project.

        The latitude at which this project is located.  # noqa: E501

        :param latitude: The latitude of this Project.  # noqa: E501
        :type: float
        """

        self._latitude = latitude

    @property
    def longitude(self):
        """Gets the longitude of this Project.  # noqa: E501

        The longitude at which this project is located.  # noqa: E501

        :return: The longitude of this Project.  # noqa: E501
        :rtype: float
        """
        return self._longitude

    @longitude.setter
    def longitude(self, longitude):
        """Sets the longitude of this Project.

        The longitude at which this project is located.  # noqa: E501

        :param longitude: The longitude of this Project.  # noqa: E501
        :type: float
        """

        self._longitude = longitude

    @property
    def developer(self):
        """Gets the developer of this Project.  # noqa: E501

        The name of the project developer.  # noqa: E501

        :return: The developer of this Project.  # noqa: E501
        :rtype: str
        """
        return self._developer

    @developer.setter
    def developer(self, developer):
        """Sets the developer of this Project.

        The name of the project developer.  # noqa: E501

        :param developer: The developer of this Project.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and developer is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `developer`, must not be `None`"
            )  # noqa: E501

        self._developer = developer

    @property
    def photos(self):
        """Gets the photos of this Project.  # noqa: E501

        An array of URLs for photos of the project.  # noqa: E501

        :return: The photos of this Project.  # noqa: E501
        :rtype: list[Photo]
        """
        return self._photos

    @photos.setter
    def photos(self, photos):
        """Sets the photos of this Project.

        An array of URLs for photos of the project.  # noqa: E501

        :param photos: The photos of this Project.  # noqa: E501
        :type: list[Photo]
        """

        self._photos = photos

    @property
    def average_price_per_tonne_cents_usd(self):
        """Gets the average_price_per_tonne_cents_usd of this Project.  # noqa: E501

        The average price per tonne in USD cents for carbon offsets supplied by this project.  # noqa: E501

        :return: The average_price_per_tonne_cents_usd of this Project.  # noqa: E501
        :rtype: int
        """
        return self._average_price_per_tonne_cents_usd

    @average_price_per_tonne_cents_usd.setter
    def average_price_per_tonne_cents_usd(self, average_price_per_tonne_cents_usd):
        """Sets the average_price_per_tonne_cents_usd of this Project.

        The average price per tonne in USD cents for carbon offsets supplied by this project.  # noqa: E501

        :param average_price_per_tonne_cents_usd: The average_price_per_tonne_cents_usd of this Project.  # noqa: E501
        :type: int
        """
        if (
            self.local_vars_configuration.client_side_validation
            and average_price_per_tonne_cents_usd is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `average_price_per_tonne_cents_usd`, must not be `None`"
            )  # noqa: E501

        self._average_price_per_tonne_cents_usd = average_price_per_tonne_cents_usd

    @property
    def remaining_mass_g(self):
        """Gets the remaining_mass_g of this Project.  # noqa: E501

        The remaining mass in grams available for purchase for this project.  # noqa: E501

        :return: The remaining_mass_g of this Project.  # noqa: E501
        :rtype: int
        """
        return self._remaining_mass_g

    @remaining_mass_g.setter
    def remaining_mass_g(self, remaining_mass_g):
        """Sets the remaining_mass_g of this Project.

        The remaining mass in grams available for purchase for this project.  # noqa: E501

        :param remaining_mass_g: The remaining_mass_g of this Project.  # noqa: E501
        :type: int
        """
        if (
            self.local_vars_configuration.client_side_validation
            and remaining_mass_g is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `remaining_mass_g`, must not be `None`"
            )  # noqa: E501

        self._remaining_mass_g = remaining_mass_g

    @property
    def verifier(self):
        """Gets the verifier of this Project.  # noqa: E501

        The name of the project verifier, when applicable. A verifier is the organization that verifies the calculations of the actual amount of greenhouse gas emissions that have been avoided or sequestered through implementation of the project.  # noqa: E501

        :return: The verifier of this Project.  # noqa: E501
        :rtype: str
        """
        return self._verifier

    @verifier.setter
    def verifier(self, verifier):
        """Sets the verifier of this Project.

        The name of the project verifier, when applicable. A verifier is the organization that verifies the calculations of the actual amount of greenhouse gas emissions that have been avoided or sequestered through implementation of the project.  # noqa: E501

        :param verifier: The verifier of this Project.  # noqa: E501
        :type: str
        """

        self._verifier = verifier

    @property
    def standard(self):
        """Gets the standard of this Project.  # noqa: E501

        An object returning the Standard associated with this project, when applicable. Standards provide guidance on GHG quantification, monitoring, and reporting. Standards can include protocols/methodologies and guidance documents.  # noqa: E501

        :return: The standard of this Project.  # noqa: E501
        :rtype: Standard
        """
        return self._standard

    @standard.setter
    def standard(self, standard):
        """Sets the standard of this Project.

        An object returning the Standard associated with this project, when applicable. Standards provide guidance on GHG quantification, monitoring, and reporting. Standards can include protocols/methodologies and guidance documents.  # noqa: E501

        :param standard: The standard of this Project.  # noqa: E501
        :type: Standard
        """

        self._standard = standard

    @property
    def sdgs(self):
        """Gets the sdgs of this Project.  # noqa: E501

        An array returning the UN Sustainable Development Goals associated with this project.  # noqa: E501

        :return: The sdgs of this Project.  # noqa: E501
        :rtype: list[Sdg]
        """
        return self._sdgs

    @sdgs.setter
    def sdgs(self, sdgs):
        """Sets the sdgs of this Project.

        An array returning the UN Sustainable Development Goals associated with this project.  # noqa: E501

        :param sdgs: The sdgs of this Project.  # noqa: E501
        :type: list[Sdg]
        """

        self._sdgs = sdgs

    @property
    def tagline(self):
        """Gets the tagline of this Project.  # noqa: E501

        A short description of the project.  # noqa: E501

        :return: The tagline of this Project.  # noqa: E501
        :rtype: str
        """
        return self._tagline

    @tagline.setter
    def tagline(self, tagline):
        """Sets the tagline of this Project.

        A short description of the project.  # noqa: E501

        :param tagline: The tagline of this Project.  # noqa: E501
        :type: str
        """

        self._tagline = tagline

    @property
    def technology_type(self):
        """Gets the technology_type of this Project.  # noqa: E501


        :return: The technology_type of this Project.  # noqa: E501
        :rtype: TechnologyType
        """
        return self._technology_type

    @technology_type.setter
    def technology_type(self, technology_type):
        """Sets the technology_type of this Project.


        :param technology_type: The technology_type of this Project.  # noqa: E501
        :type: TechnologyType
        """
        if (
            self.local_vars_configuration.client_side_validation
            and technology_type is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `technology_type`, must not be `None`"
            )  # noqa: E501

        self._technology_type = technology_type

    @property
    def highlights(self):
        """Gets the highlights of this Project.  # noqa: E501

        An array of objects containing the highlight's slug, title, and a URL for the corresponding icon.  A highlight's title is a short string that spotlights a characteristic about the project.  # noqa: E501

        :return: The highlights of this Project.  # noqa: E501
        :rtype: list[Highlight]
        """
        return self._highlights

    @highlights.setter
    def highlights(self, highlights):
        """Sets the highlights of this Project.

        An array of objects containing the highlight's slug, title, and a URL for the corresponding icon.  A highlight's title is a short string that spotlights a characteristic about the project.  # noqa: E501

        :param highlights: The highlights of this Project.  # noqa: E501
        :type: list[Highlight]
        """
        if (
            self.local_vars_configuration.client_side_validation and highlights is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `highlights`, must not be `None`"
            )  # noqa: E501

        self._highlights = highlights

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Project):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Project):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa
"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from patch_api.models.allocation import Allocation
from patch_api.models.create_bitcoin_estimate_request import (
    CreateBitcoinEstimateRequest,
)
from patch_api.models.create_ethereum_estimate_request import (
    CreateEthereumEstimateRequest,
)
from patch_api.models.create_flight_estimate_request import CreateFlightEstimateRequest
from patch_api.models.create_hotel_estimate_request import CreateHotelEstimateRequest
from patch_api.models.create_mass_estimate_request import CreateMassEstimateRequest
from patch_api.models.create_order_request import CreateOrderRequest
from patch_api.models.create_shipping_estimate_request import (
    CreateShippingEstimateRequest,
)
from patch_api.models.create_success_response import CreateSuccessResponse
from patch_api.models.create_vehicle_estimate_request import (
    CreateVehicleEstimateRequest,
)
from patch_api.models.error_response import ErrorResponse
from patch_api.models.estimate import Estimate
from patch_api.models.estimate_list_response import EstimateListResponse
from patch_api.models.estimate_response import EstimateResponse
from patch_api.models.highlight import Highlight
from patch_api.models.meta_index_object import MetaIndexObject
from patch_api.models.order import Order
from patch_api.models.order_list_response import OrderListResponse
from patch_api.models.order_response import OrderResponse
from patch_api.models.parent_technology_type import ParentTechnologyType
from patch_api.models.photo import Photo
from patch_api.models.project import Project
from patch_api.models.project_list_response import ProjectListResponse
from patch_api.models.project_response import ProjectResponse
from patch_api.models.sdg import Sdg
from patch_api.models.standard import Standard
from patch_api.models.technology_type import TechnologyType
from patch_api.models.technology_type_list_response import TechnologyTypeListResponse



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class Estimate(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "str",
        "production": "bool",
        "type": "str",
        "mass_g": "int",
        "order": "Order",
    }

    attribute_map = {
        "id": "id",
        "production": "production",
        "type": "type",
        "mass_g": "mass_g",
        "order": "order",
    }

    def __init__(
        self,
        id=None,
        production=None,
        type=None,
        mass_g=None,
        order=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """Estimate - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._production = None
        self._type = None
        self._mass_g = None
        self._order = None
        self.discriminator = None

        self.id = id
        self.production = production
        self.type = type
        if mass_g is not None:
            self.mass_g = mass_g
        self.order = order

    @property
    def id(self):
        """Gets the id of this Estimate.  # noqa: E501

        A unique uid for the record. UIDs will be prepended by est_prod or est_test depending on the mode it was created in.  # noqa: E501

        :return: The id of this Estimate.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Estimate.

        A unique uid for the record. UIDs will be prepended by est_prod or est_test depending on the mode it was created in.  # noqa: E501

        :param id: The id of this Estimate.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and id is None
        ):  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def production(self):
        """Gets the production of this Estimate.  # noqa: E501

        A boolean indicating if this estimate is a production or test mode estimate.  # noqa: E501

        :return: The production of this Estimate.  # noqa: E501
        :rtype: bool
        """
        return self._production

    @production.setter
    def production(self, production):
        """Sets the production of this Estimate.

        A boolean indicating if this estimate is a production or test mode estimate.  # noqa: E501

        :param production: The production of this Estimate.  # noqa: E501
        :type: bool
        """
        if (
            self.local_vars_configuration.client_side_validation and production is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `production`, must not be `None`"
            )  # noqa: E501

        self._production = production

    @property
    def type(self):
        """Gets the type of this Estimate.  # noqa: E501

        The type of estimate. Available types are mass, flight, shipping, vehicle, and crypto.  # noqa: E501

        :return: The type of this Estimate.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Estimate.

        The type of estimate. Available types are mass, flight, shipping, vehicle, and crypto.  # noqa: E501

        :param type: The type of this Estimate.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and type is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `type`, must not be `None`"
            )  # noqa: E501

        self._type = type

    @property
    def mass_g(self):
        """Gets the mass_g of this Estimate.  # noqa: E501

        The estimated mass in grams for this estimate.  # noqa: E501

        :return: The mass_g of this Estimate.  # noqa: E501
        :rtype: int
        """
        return self._mass_g

    @mass_g.setter
    def mass_g(self, mass_g):
        """Sets the mass_g of this Estimate.

        The estimated mass in grams for this estimate.  # noqa: E501

        :param mass_g: The mass_g of this Estimate.  # noqa: E501
        :type: int
        """

        self._mass_g = mass_g

    @property
    def order(self):
        """Gets the order of this Estimate.  # noqa: E501

        An object returning the order associated with this estimate. See the [Order section](/?id=orders) for the full schema.  # noqa: E501

        :return: The order of this Estimate.  # noqa: E501
        :rtype: Order
        """
        return self._order

    @order.setter
    def order(self, order):
        """Sets the order of this Estimate.

        An object returning the order associated with this estimate. See the [Order section](/?id=orders) for the full schema.  # noqa: E501

        :param order: The order of this Estimate.  # noqa: E501
        :type: Order
        """

        self._order = order

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Estimate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Estimate):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class ProjectListResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "success": "bool",
        "error": "object",
        "data": "list[Project]",
        "meta": "MetaIndexObject",
    }

    attribute_map = {
        "success": "success",
        "error": "error",
        "data": "data",
        "meta": "meta",
    }

    def __init__(
        self,
        success=None,
        error=None,
        data=None,
        meta=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """ProjectListResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._success = None
        self._error = None
        self._data = None
        self._meta = None
        self.discriminator = None

        self.success = success
        self.error = error
        self.data = data
        self.meta = meta

    @property
    def success(self):
        """Gets the success of this ProjectListResponse.  # noqa: E501


        :return: The success of this ProjectListResponse.  # noqa: E501
        :rtype: bool
        """
        return self._success

    @success.setter
    def success(self, success):
        """Sets the success of this ProjectListResponse.


        :param success: The success of this ProjectListResponse.  # noqa: E501
        :type: bool
        """
        if (
            self.local_vars_configuration.client_side_validation and success is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `success`, must not be `None`"
            )  # noqa: E501

        self._success = success

    @property
    def error(self):
        """Gets the error of this ProjectListResponse.  # noqa: E501


        :return: The error of this ProjectListResponse.  # noqa: E501
        :rtype: object
        """
        return self._error

    @error.setter
    def error(self, error):
        """Sets the error of this ProjectListResponse.


        :param error: The error of this ProjectListResponse.  # noqa: E501
        :type: object
        """

        self._error = error

    @property
    def data(self):
        """Gets the data of this ProjectListResponse.  # noqa: E501


        :return: The data of this ProjectListResponse.  # noqa: E501
        :rtype: list[Project]
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this ProjectListResponse.


        :param data: The data of this ProjectListResponse.  # noqa: E501
        :type: list[Project]
        """
        if (
            self.local_vars_configuration.client_side_validation and data is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `data`, must not be `None`"
            )  # noqa: E501

        self._data = data

    @property
    def meta(self):
        """Gets the meta of this ProjectListResponse.  # noqa: E501


        :return: The meta of this ProjectListResponse.  # noqa: E501
        :rtype: MetaIndexObject
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """Sets the meta of this ProjectListResponse.


        :param meta: The meta of this ProjectListResponse.  # noqa: E501
        :type: MetaIndexObject
        """
        if (
            self.local_vars_configuration.client_side_validation and meta is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `meta`, must not be `None`"
            )  # noqa: E501

        self._meta = meta

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectListResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProjectListResponse):
            return True

        return self.to_dict() != other.to_dict()



from __future__ import absolute_import

from patch_api.configuration import Configuration
from patch_api import rest

import unittest
import os

from patch_api.api_client import ApiClient


class TestRESTClientObject(unittest.TestCase):
    def test_recursive_urlencode(self):
        """Test that the correct query params get encoded"""

        api_key = os.environ.get("SANDBOX_API_KEY")
        configuration = Configuration()
        client = rest.RESTClientObject(api_key, configuration)

        encoded_params = client.recursive_urlencode({})
        self.assertEqual(encoded_params, "")

        encoded_params = client.recursive_urlencode({"page": 1})
        self.assertEqual(encoded_params, "page=1")

        encoded_params = client.recursive_urlencode({"string": "value"})
        self.assertEqual(encoded_params, "string=value")

        encoded_params = client.recursive_urlencode(
            {"multiple": "values", "top": "level"}
        )
        self.assertEqual(encoded_params, "multiple=values&top=level")

        encoded_params = client.recursive_urlencode({"metadata": {"some": "arg"}})
        self.assertEqual(encoded_params, "metadata[some]=arg")

        encoded_params = client.recursive_urlencode(
            {"deeply": {"nested": {"hash": {"of": "args"}}}}
        )
        self.assertEqual(encoded_params, "deeply[nested][hash][of]=args")

    def test_encoded_query_params(self):
        """Test that the correct query params get encoded"""

        api_key = os.environ.get("SANDBOX_API_KEY")
        configuration = Configuration()
        client = rest.RESTClientObject(api_key, configuration)

        encoded_params = client.encoded_query_params([])
        self.assertEqual(encoded_params, "")

        encoded_params = client.encoded_query_params([("mass_g", 100)])
        self.assertEqual(encoded_params, "?mass_g=100")

        encoded_params = client.encoded_query_params(
            [("mass_g", 100), ("metadata", {"external_id": "abc-123"})]
        )
        self.assertEqual(encoded_params, "?metadata[external_id]=abc-123&mass_g=100")



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class Highlight(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"slug": "str", "title": "str", "icon_url": "str"}

    attribute_map = {"slug": "slug", "title": "title", "icon_url": "icon_url"}

    def __init__(
        self, slug=None, title=None, icon_url=None, local_vars_configuration=None
    ):  # noqa: E501
        """Highlight - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._slug = None
        self._title = None
        self._icon_url = None
        self.discriminator = None

        self.slug = slug
        self.title = title
        self.icon_url = icon_url

    @property
    def slug(self):
        """Gets the slug of this Highlight.  # noqa: E501

        A unique identifier for each highlight.  # noqa: E501

        :return: The slug of this Highlight.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this Highlight.

        A unique identifier for each highlight.  # noqa: E501

        :param slug: The slug of this Highlight.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and slug is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `slug`, must not be `None`"
            )  # noqa: E501

        self._slug = slug

    @property
    def title(self):
        """Gets the title of this Highlight.  # noqa: E501

        A short string that spotlights a characteristic about the project.  # noqa: E501

        :return: The title of this Highlight.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this Highlight.

        A short string that spotlights a characteristic about the project.  # noqa: E501

        :param title: The title of this Highlight.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and title is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `title`, must not be `None`"
            )  # noqa: E501

        self._title = title

    @property
    def icon_url(self):
        """Gets the icon_url of this Highlight.  # noqa: E501

        A URL for the corresponding icon.  # noqa: E501

        :return: The icon_url of this Highlight.  # noqa: E501
        :rtype: str
        """
        return self._icon_url

    @icon_url.setter
    def icon_url(self, icon_url):
        """Sets the icon_url of this Highlight.

        A URL for the corresponding icon.  # noqa: E501

        :param icon_url: The icon_url of this Highlight.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and icon_url is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `icon_url`, must not be `None`"
            )  # noqa: E501

        self._icon_url = icon_url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Highlight):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Highlight):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from patch_api.exceptions import ApiTypeError, ApiValueError


class TechnologyTypesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    ALLOWED_QUERY_PARAMS = [
        "mass_g",
        "total_price_cents_usd",
        "project_id",
        "page",
        "distance_m",
        "transportation_method",
        "package_mass_g",
        "create_order",
        "model",
        "make",
        "year",
        "transaction_value_btc_sats",
        "transaction_value_eth_gwei",
        "gas_used",
        "average_daily_balance_btc_sats",
        "average_daily_balance_eth_gwei",
        "timestamp",
        "origin_aiport",
        "destination_aiport",
        "aircraft_code",
        "cabin_class",
        "passenger_count",
        "state",
        "country_code",
        "city",
        "region",
        "star_rating",
        "number_of_nights",
        "number_of_rooms",
        "vintage_year",
    ]

    def __init__(self, api_client=None):
        self.api_client = api_client

    def retrieve_technology_types(self, **kwargs):  # noqa: E501
        """Retrieves the list of technology_types  # noqa: E501

        Retrieves a list of all technology_types.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_technology_types(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: TechnologyTypeListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.retrieve_technology_types_with_http_info(**kwargs)  # noqa: E501

    def retrieve_technology_types_with_http_info(self, **kwargs):  # noqa: E501
        """Retrieves the list of technology_types  # noqa: E501

        Retrieves a list of all technology_types.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_technology_types_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(TechnologyTypeListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = []  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")
        all_params.append("mass_g")
        all_params.append("total_price_cents_usd")
        all_params.append("project_id")
        all_params.append("metadata")
        all_params.append("distance_m")
        all_params.append("transportation_method")
        all_params.append("package_mass_g")
        all_params.append("create_order")
        all_params.append("make")
        all_params.append("model")
        all_params.append("year")
        all_params.append("transaction_value_btc_sats")
        all_params.append("transaction_value_eth_gwei")
        all_params.append("gas_used")
        all_params.append("transaction_value_btc_sats")
        all_params.append("average_daily_balance_btc_sats")
        all_params.append("average_daily_balance_eth_gwei")
        all_params.append("timestamp")
        all_params.append("origin_airport")
        all_params.append("destination_airport")
        all_params.append("aircraft_code")
        all_params.append("cabin_class")
        all_params.append("passenger_count")
        all_params.append("state")
        all_params.append("country_code")
        all_params.append("city")
        all_params.append("region")
        all_params.append("star_rating")
        all_params.append("number_of_nights")
        all_params.append("number_of_rooms")
        all_params.append("vintage_year")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_technology_types" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        # do not add duplicate keys to query_params list
        existing_keys = []
        for param in query_params:
            existing_keys.append(param[0])

        for key in kwargs:
            if key not in existing_keys:
                query_params.append([key, kwargs.get(key)])

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["bearer_auth"]  # noqa: E501

        return self.api_client.call_api(
            "/v1/projects/technology_types",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="TechnologyTypeListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class Order(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "str",
        "created_at": "datetime",
        "mass_g": "int",
        "production": "bool",
        "state": "str",
        "allocation_state": "str",
        "price_cents_usd": "int",
        "patch_fee_cents_usd": "int",
        "allocations": "list[Allocation]",
        "registry_url": "str",
        "metadata": "object",
    }

    attribute_map = {
        "id": "id",
        "created_at": "created_at",
        "mass_g": "mass_g",
        "production": "production",
        "state": "state",
        "allocation_state": "allocation_state",
        "price_cents_usd": "price_cents_usd",
        "patch_fee_cents_usd": "patch_fee_cents_usd",
        "allocations": "allocations",
        "registry_url": "registry_url",
        "metadata": "metadata",
    }

    def __init__(
        self,
        id=None,
        created_at=None,
        mass_g=None,
        production=None,
        state=None,
        allocation_state=None,
        price_cents_usd=None,
        patch_fee_cents_usd=None,
        allocations=None,
        registry_url=None,
        metadata=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """Order - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._created_at = None
        self._mass_g = None
        self._production = None
        self._state = None
        self._allocation_state = None
        self._price_cents_usd = None
        self._patch_fee_cents_usd = None
        self._allocations = None
        self._registry_url = None
        self._metadata = None
        self.discriminator = None

        self.id = id
        if created_at is not None:
            self.created_at = created_at
        self.mass_g = mass_g
        self.production = production
        self.state = state
        self.allocation_state = allocation_state
        self.price_cents_usd = price_cents_usd
        self.patch_fee_cents_usd = patch_fee_cents_usd
        if allocations is not None:
            self.allocations = allocations
        if registry_url is not None:
            self.registry_url = registry_url
        self.metadata = metadata

    @property
    def id(self):
        """Gets the id of this Order.  # noqa: E501

        A unique uid for the record. UIDs will be prepended by ord_prod or ord_test depending on the mode it was created in.  # noqa: E501

        :return: The id of this Order.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Order.

        A unique uid for the record. UIDs will be prepended by ord_prod or ord_test depending on the mode it was created in.  # noqa: E501

        :param id: The id of this Order.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and id is None
        ):  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def created_at(self):
        """Gets the created_at of this Order.  # noqa: E501

        The timestamp at which the order was created  # noqa: E501

        :return: The created_at of this Order.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Order.

        The timestamp at which the order was created  # noqa: E501

        :param created_at: The created_at of this Order.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def mass_g(self):
        """Gets the mass_g of this Order.  # noqa: E501

        The amount of carbon offsets in grams purchased through this order.  # noqa: E501

        :return: The mass_g of this Order.  # noqa: E501
        :rtype: int
        """
        return self._mass_g

    @mass_g.setter
    def mass_g(self, mass_g):
        """Sets the mass_g of this Order.

        The amount of carbon offsets in grams purchased through this order.  # noqa: E501

        :param mass_g: The mass_g of this Order.  # noqa: E501
        :type: int
        """
        if (
            self.local_vars_configuration.client_side_validation and mass_g is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must not be `None`"
            )  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and mass_g is not None
            and mass_g > 100000000000
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must be a value less than or equal to `100000000000`"
            )  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and mass_g is not None
            and mass_g < 0
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `mass_g`, must be a value greater than or equal to `0`"
            )  # noqa: E501

        self._mass_g = mass_g

    @property
    def production(self):
        """Gets the production of this Order.  # noqa: E501

        A boolean indicating if this order is a production or test mode order.  # noqa: E501

        :return: The production of this Order.  # noqa: E501
        :rtype: bool
        """
        return self._production

    @production.setter
    def production(self, production):
        """Sets the production of this Order.

        A boolean indicating if this order is a production or test mode order.  # noqa: E501

        :param production: The production of this Order.  # noqa: E501
        :type: bool
        """
        if (
            self.local_vars_configuration.client_side_validation and production is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `production`, must not be `None`"
            )  # noqa: E501

        self._production = production

    @property
    def state(self):
        """Gets the state of this Order.  # noqa: E501

        The current state of the order.  # noqa: E501

        :return: The state of this Order.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this Order.

        The current state of the order.  # noqa: E501

        :param state: The state of this Order.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and state is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `state`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "draft",
            "placed",
            "processing",
            "complete",
            "cancelled",
        ]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and state not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `state` ({0}), must be one of {1}".format(  # noqa: E501
                    state, allowed_values
                )
            )

        self._state = state

    @property
    def allocation_state(self):
        """Gets the allocation_state of this Order.  # noqa: E501

        DEPRECATED. Indicates if the order has been fully allocated to projects.  # noqa: E501

        :return: The allocation_state of this Order.  # noqa: E501
        :rtype: str
        """
        return self._allocation_state

    @allocation_state.setter
    def allocation_state(self, allocation_state):
        """Sets the allocation_state of this Order.

        DEPRECATED. Indicates if the order has been fully allocated to projects.  # noqa: E501

        :param allocation_state: The allocation_state of this Order.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation
            and allocation_state is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `allocation_state`, must not be `None`"
            )  # noqa: E501
        allowed_values = ["pending", "allocated"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and allocation_state not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `allocation_state` ({0}), must be one of {1}".format(  # noqa: E501
                    allocation_state, allowed_values
                )
            )

        self._allocation_state = allocation_state

    @property
    def price_cents_usd(self):
        """Gets the price_cents_usd of this Order.  # noqa: E501

        The total price in cents USD of the carbon offsets purchased through this order.  # noqa: E501

        :return: The price_cents_usd of this Order.  # noqa: E501
        :rtype: int
        """
        return self._price_cents_usd

    @price_cents_usd.setter
    def price_cents_usd(self, price_cents_usd):
        """Sets the price_cents_usd of this Order.

        The total price in cents USD of the carbon offsets purchased through this order.  # noqa: E501

        :param price_cents_usd: The price_cents_usd of this Order.  # noqa: E501
        :type: int
        """

        self._price_cents_usd = price_cents_usd

    @property
    def patch_fee_cents_usd(self):
        """Gets the patch_fee_cents_usd of this Order.  # noqa: E501

        The Patch Fee in cents USD for this order.  # noqa: E501

        :return: The patch_fee_cents_usd of this Order.  # noqa: E501
        :rtype: int
        """
        return self._patch_fee_cents_usd

    @patch_fee_cents_usd.setter
    def patch_fee_cents_usd(self, patch_fee_cents_usd):
        """Sets the patch_fee_cents_usd of this Order.

        The Patch Fee in cents USD for this order.  # noqa: E501

        :param patch_fee_cents_usd: The patch_fee_cents_usd of this Order.  # noqa: E501
        :type: int
        """

        self._patch_fee_cents_usd = patch_fee_cents_usd

    @property
    def allocations(self):
        """Gets the allocations of this Order.  # noqa: E501

        DEPRECATED. An array containing the inventory allocations for this order.  # noqa: E501

        :return: The allocations of this Order.  # noqa: E501
        :rtype: list[Allocation]
        """
        return self._allocations

    @allocations.setter
    def allocations(self, allocations):
        """Sets the allocations of this Order.

        DEPRECATED. An array containing the inventory allocations for this order.  # noqa: E501

        :param allocations: The allocations of this Order.  # noqa: E501
        :type: list[Allocation]
        """

        self._allocations = allocations

    @property
    def registry_url(self):
        """Gets the registry_url of this Order.  # noqa: E501

        The url of this order in the public registry.  # noqa: E501

        :return: The registry_url of this Order.  # noqa: E501
        :rtype: str
        """
        return self._registry_url

    @registry_url.setter
    def registry_url(self, registry_url):
        """Sets the registry_url of this Order.

        The url of this order in the public registry.  # noqa: E501

        :param registry_url: The registry_url of this Order.  # noqa: E501
        :type: str
        """

        self._registry_url = registry_url

    @property
    def metadata(self):
        """Gets the metadata of this Order.  # noqa: E501

        An optional JSON object containing metadata for this order.  # noqa: E501

        :return: The metadata of this Order.  # noqa: E501
        :rtype: object
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """Sets the metadata of this Order.

        An optional JSON object containing metadata for this order.  # noqa: E501

        :param metadata: The metadata of this Order.  # noqa: E501
        :type: object
        """
        if (
            self.local_vars_configuration.client_side_validation and metadata is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `metadata`, must not be `None`"
            )  # noqa: E501

        self._metadata = metadata

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Order):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Order):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.20.0"

# import ApiClient
from patch_api.api_client import ApiClient
from patch_api.configuration import Configuration
from patch_api.exceptions import OpenApiException
from patch_api.exceptions import ApiTypeError
from patch_api.exceptions import ApiValueError
from patch_api.exceptions import ApiKeyError
from patch_api.exceptions import ApiException

from patch_api.api import *

from patch_api.models import *



"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "patch-api"
VERSION = "1.20.0"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "urllib3 >= 1.25.3",
    "python-dateutil",
    "certifi",
]

setup(
    name=NAME,
    version=VERSION,
    description="Patch API V1",
    author="Developer Support",
    author_email="engineering@usepatch.com",
    url="",
    keywords=["OpenAPI", "OpenAPI-Generator", "Patch API V1"],
    python_requires=">=3.6",
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description="""\
    The core API used to integrate with Patch&#39;s service  # noqa: E501
    """,
)



# coding: utf-8

"""
    Patch API V1

    The core API used to integrate with Patch's service  # noqa: E501

    The version of the OpenAPI document: v1
    Contact: engineering@usepatch.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from patch_api.configuration import Configuration


class CreateHotelEstimateRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "country_code": "str",
        "city": "str",
        "region": "str",
        "star_rating": "int",
        "number_of_nights": "int",
        "number_of_rooms": "int",
        "project_id": "str",
        "create_order": "bool",
    }

    attribute_map = {
        "country_code": "country_code",
        "city": "city",
        "region": "region",
        "star_rating": "star_rating",
        "number_of_nights": "number_of_nights",
        "number_of_rooms": "number_of_rooms",
        "project_id": "project_id",
        "create_order": "create_order",
    }

    def __init__(
        self,
        country_code=None,
        city=None,
        region=None,
        star_rating=None,
        number_of_nights=None,
        number_of_rooms=None,
        project_id=None,
        create_order=False,
        local_vars_configuration=None,
    ):  # noqa: E501
        """CreateHotelEstimateRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._country_code = None
        self._city = None
        self._region = None
        self._star_rating = None
        self._number_of_nights = None
        self._number_of_rooms = None
        self._project_id = None
        self._create_order = None
        self.discriminator = None

        self.country_code = country_code
        if city is not None:
            self.city = city
        if region is not None:
            self.region = region
        if star_rating is not None:
            self.star_rating = star_rating
        if number_of_nights is not None:
            self.number_of_nights = number_of_nights
        if number_of_rooms is not None:
            self.number_of_rooms = number_of_rooms
        self.project_id = project_id
        self.create_order = create_order

    @property
    def country_code(self):
        """Gets the country_code of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The country_code of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: str
        """
        return self._country_code

    @country_code.setter
    def country_code(self, country_code):
        """Sets the country_code of this CreateHotelEstimateRequest.


        :param country_code: The country_code of this CreateHotelEstimateRequest.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation
            and country_code is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `country_code`, must not be `None`"
            )  # noqa: E501

        self._country_code = country_code

    @property
    def city(self):
        """Gets the city of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The city of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: str
        """
        return self._city

    @city.setter
    def city(self, city):
        """Sets the city of this CreateHotelEstimateRequest.


        :param city: The city of this CreateHotelEstimateRequest.  # noqa: E501
        :type: str
        """

        self._city = city

    @property
    def region(self):
        """Gets the region of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The region of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this CreateHotelEstimateRequest.


        :param region: The region of this CreateHotelEstimateRequest.  # noqa: E501
        :type: str
        """

        self._region = region

    @property
    def star_rating(self):
        """Gets the star_rating of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The star_rating of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: int
        """
        return self._star_rating

    @star_rating.setter
    def star_rating(self, star_rating):
        """Sets the star_rating of this CreateHotelEstimateRequest.


        :param star_rating: The star_rating of this CreateHotelEstimateRequest.  # noqa: E501
        :type: int
        """

        self._star_rating = star_rating

    @property
    def number_of_nights(self):
        """Gets the number_of_nights of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The number_of_nights of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: int
        """
        return self._number_of_nights

    @number_of_nights.setter
    def number_of_nights(self, number_of_nights):
        """Sets the number_of_nights of this CreateHotelEstimateRequest.


        :param number_of_nights: The number_of_nights of this CreateHotelEstimateRequest.  # noqa: E501
        :type: int
        """

        self._number_of_nights = number_of_nights

    @property
    def number_of_rooms(self):
        """Gets the number_of_rooms of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The number_of_rooms of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: int
        """
        return self._number_of_rooms

    @number_of_rooms.setter
    def number_of_rooms(self, number_of_rooms):
        """Sets the number_of_rooms of this CreateHotelEstimateRequest.


        :param number_of_rooms: The number_of_rooms of this CreateHotelEstimateRequest.  # noqa: E501
        :type: int
        """

        self._number_of_rooms = number_of_rooms

    @property
    def project_id(self):
        """Gets the project_id of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The project_id of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this CreateHotelEstimateRequest.


        :param project_id: The project_id of this CreateHotelEstimateRequest.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def create_order(self):
        """Gets the create_order of this CreateHotelEstimateRequest.  # noqa: E501


        :return: The create_order of this CreateHotelEstimateRequest.  # noqa: E501
        :rtype: bool
        """
        return self._create_order

    @create_order.setter
    def create_order(self, create_order):
        """Sets the create_order of this CreateHotelEstimateRequest.


        :param create_order: The create_order of this CreateHotelEstimateRequest.  # noqa: E501
        :type: bool
        """

        self._create_order = create_order

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateHotelEstimateRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateHotelEstimateRequest):
            return True

        return self.to_dict() != other.to_dict()
